@@ -1,14 +1,9 @@
-import triton
-from op_tests.op_benchmarks.triton.utils.benchmark_utils import (
-    get_model_configs,
-    print_vgpr,
-)
 import torch
 import sys
 import warnings
 import argparse
 import itertools
-
+import triton
 from aiter.ops.triton.mha import (
     flash_attn_func,
     flash_attn_fp8_func,
@@ -21,6 +16,11 @@ from aiter.test_mha_common import (
     generate_qkv,
 )
 from op_tests.op_benchmarks.triton.utils.argparse import get_parser
+from op_tests.op_benchmarks.triton.utils.benchmark_utils import (
+    get_model_configs,
+    print_vgpr,
+    get_caller_name_no_ext,
+)
 
 
 def nonvarlen_benchmark_configs():
@@ -120,7 +120,7 @@ def create_benchmark_configs(custom, args):
     varlen = args.layout == "thd"
 
     configs = []
-    plot_name = f"fused-attention-{mode}-D_HEAD-{head_size}-layout-{args.layout}-fp8-{args.fp8}-causal-{causal}"
+    plot_name = get_caller_name_no_ext()
     extra_args = {"D_HEAD": head_size, "dtype": dtype, "causal": causal, "mode": mode}
 
     if custom:
@@ -464,9 +464,9 @@ def run_benchmark(custom, args):
         elif "TFLOPS" in provider:
             return total_flops / ms * 1e-9
         else:  # GB/s
-            return mem / ms * 1e-3
+            return mem / ms * 1e-6
 
-    bench_mha.run(save_path=".", print_data=True, show_plots=False)
+    bench_mha.run(save_path="." if args.o else None, print_data=True)
 
 
 def supported_layouts():
@@ -513,12 +513,6 @@ def parse_args():
     parser.add_argument("-bench_torch", action="store_true", default=False)
     parser.add_argument("-fused_bwd", action="store_true", default=False)
     parser.add_argument("-print_vgpr", action="store_true", default=False)
-    parser.add_argument(
-        "-return_all",
-        action="store_true",
-        default=False,
-        help="Prints TFLOPS, walltime, bandwidth.",
-    )
     parser.add_argument(
         "-test_mode",
         action="store_true",
@@ -527,7 +521,14 @@ def parse_args():
     )
 
     parser.add_argument("--layout", type=str, default=None, help=supported_layouts())
-
+    parser.add_argument(
+        "-metric",
+        nargs="?",
+        const="throughput",
+        choices=["time", "throughput", "bandwidth"],
+        default=None,
+        help="Metrics for the kernel benchmark.",
+    )
     parser.add_argument(
         "-persistent",
         nargs="?",
@@ -536,6 +537,9 @@ def parse_args():
         default=None,
         help="Enable persistent kernels. Use '-persistent dynamic' for dynamic scheduling of the tiles.",
     )
+    parser.add_argument(
+        "-o", action="store_true", help="Write performance results to CSV file"
+    )
     return parser.parse_args()
 
 
@@ -604,7 +608,7 @@ def main():
         def fun():
             return run_benchmark(custom_config, args)
 
-        print_vgpr(fun, "fused-attention")
+        print_vgpr(fun, get_caller_name_no_ext())
         return 0
 
     run_benchmark(custom_config, args)
