@@ -10,6 +10,10 @@ from aiter.ops.triton.quant import dynamic_per_tensor_quant_fp8_i8
 from aiter.ops.triton.activation import _gelu_tanh
 from aiter.ops.triton.utils.pid_preprocessing import pid_grid, remap_xcd
 from aiter.ops.triton.utils.moe_common import _write_zeros_to_output
+from aiter.ops.triton.utils.logger import AiterTritonLogger
+from aiter.ops.triton.utils.arch_info import get_num_xcds
+
+_LOGGER = AiterTritonLogger()
 
 # Source:
 # MoE Kernel adapted from VLLM
@@ -96,6 +100,7 @@ def _fused_moe_kernel(
     compute_type: tl.constexpr,
     use_fp8_w8a8: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -131,7 +136,6 @@ def _fused_moe_kernel(
 
     num_pid_m = tl.cdiv(num_tokens_post_padded, BLOCK_SIZE_M)
     num_pid_n = tl.cdiv(N, BLOCK_SIZE_N)
-    NUM_XCDS: tl.constexpr = 8
 
     GRID_MN = num_pid_n * num_pid_m
     if pid < GRID_MN:
@@ -316,6 +320,7 @@ def _fused_moe_persistent_kernel(
     compute_type: tl.constexpr,
     use_fp8_w8a8: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -347,7 +352,6 @@ def _fused_moe_persistent_kernel(
     # -----------------------------------------------------------
     # Simply compute how many iterations each persistent block needs to do
     start_pid = tl.program_id(axis=0)
-    NUM_XCDS: tl.constexpr = 8
 
     num_tokens_post_padded = tl.load(num_tokens_post_padded_ptr)
 
@@ -493,6 +497,11 @@ def fused_moe_gelu(
     """
     #TODO: Add doc
     """
+    _LOGGER.info(
+        f"FUSED_MOE_GELU:  A={tuple(A.shape)}  B={tuple(B.shape)}  C={tuple(C.shape)}  topk_weights-{tuple(topk_weights.shape)}"
+        + f"sorted_token_ids={tuple(sorted_token_ids.shape)} expert_ids={tuple(expert_ids.shape)}"
+        + f"num_tokens_post_padded={tuple(num_tokens_post_padded.shape)} top_k={top_k} "
+    )
     assert topk_weights.stride(1) == 1
     assert sorted_token_ids.stride(0) == 1
 
@@ -572,6 +581,7 @@ def fused_moe_gelu(
             compute_type=compute_type,
             use_fp8_w8a8=use_fp8_w8a8,
             use_int8_w8a16=use_int8_w8a16,
+            NUM_XCDS=get_num_xcds(),
             **config,
         )
     else:
@@ -613,5 +623,6 @@ def fused_moe_gelu(
             compute_type=compute_type,
             use_fp8_w8a8=use_fp8_w8a8,
             use_int8_w8a16=use_int8_w8a16,
+            NUM_XCDS=get_num_xcds(),
             **config,
         )
