@@ -10,6 +10,10 @@ from aiter.ops.triton.activation import _silu_exp2
 from aiter.ops.triton.quant import dynamic_per_tensor_quant_fp8_i8
 from aiter.ops.triton.utils.pid_preprocessing import pid_grid, remap_xcd
 from aiter.ops.triton.utils.moe_common import _write_zeros_to_output
+from aiter.ops.triton.utils.logger import AiterTritonLogger
+from aiter.ops.triton.utils.arch_info import get_num_xcds
+
+_LOGGER = AiterTritonLogger()
 
 # Source:
 # MoE Kernel adapted from VLLM
@@ -96,6 +100,7 @@ def _fused_moe_silu_kernel_gptq_awq(
     has_zp: tl.constexpr,
     use_int4_w4a16: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -131,7 +136,6 @@ def _fused_moe_silu_kernel_gptq_awq(
 
     num_pid_m = tl.cdiv(num_tokens_post_padded, BLOCK_SIZE_M)
     num_pid_n = tl.cdiv(N, BLOCK_SIZE_N)
-    NUM_XCDS: tl.constexpr = 8
 
     GRID_MN = num_pid_n * num_pid_m
     if pid < GRID_MN:
@@ -293,7 +297,6 @@ def _fused_moe_silu_kernel_gptq_awq(
     silu_acc, mul_acc = (
         accumulator.to(tl.float32).reshape(BLOCK_SIZE_M, BLOCK_SIZE_HALF, 2).split()
     )
-    # silu_acc = silu_acc / (1.0 + tl.exp2(-(silu_acc * 1.44269504089)))
     silu_acc = _silu_exp2(silu_acc)
     accumulator = (silu_acc * mul_acc).to(compute_type)
 
@@ -359,6 +362,7 @@ def _fused_moe_persistent_silu_kernel_gptq_awq(
     has_zp: tl.constexpr,
     use_int4_w4a16: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -390,7 +394,6 @@ def _fused_moe_persistent_silu_kernel_gptq_awq(
     # Map program ids `pid` to the block of C it should compute.
     # This is done in a grouped ordering to promote L2 data reuse.
     start_pid = tl.program_id(axis=0)
-    NUM_XCDS: tl.constexpr = 8
     # Load tile-invariant runtime constant
     num_tokens_post_padded = tl.load(num_tokens_post_padded_ptr)
 
@@ -539,7 +542,7 @@ def _fused_moe_persistent_silu_kernel_gptq_awq(
         silu_acc, mul_acc = (
             accumulator.to(tl.float32).reshape(BLOCK_SIZE_M, BLOCK_SIZE_HALF, 2).split()
         )
-        silu_acc = silu_acc / (1.0 + tl.exp2(-(silu_acc * 1.44269504089)))
+        silu_acc = _silu_exp2(silu_acc)
         accumulator = (silu_acc * mul_acc).to(compute_type)
 
         # -----------------------------------------------------------
@@ -604,6 +607,7 @@ def _fused_moe_silu_kernel(
     compute_type: tl.constexpr,
     use_fp8_w8a8: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -639,7 +643,6 @@ def _fused_moe_silu_kernel(
 
     num_pid_m = tl.cdiv(num_tokens_post_padded, BLOCK_SIZE_M)
     num_pid_n = tl.cdiv(N, BLOCK_SIZE_N)
-    NUM_XCDS: tl.constexpr = 8
 
     GRID_MN = num_pid_n * num_pid_m
     if pid < GRID_MN:
@@ -772,7 +775,7 @@ def _fused_moe_silu_kernel(
     silu_acc, mul_acc = (
         accumulator.to(tl.float32).reshape(BLOCK_SIZE_M, BLOCK_SIZE_HALF, 2).split()
     )
-    silu_acc = silu_acc / (1.0 + tl.exp2(-(silu_acc * 1.44269504089)))
+    silu_acc = _silu_exp2(silu_acc)
     accumulator = (silu_acc * mul_acc).to(compute_type)
 
     # -----------------------------------------------------------
@@ -836,6 +839,7 @@ def _fused_moe_persistent_silu_kernel(
     compute_type: tl.constexpr,
     use_fp8_w8a8: tl.constexpr,
     use_int8_w8a16: tl.constexpr,
+    NUM_XCDS: tl.constexpr,
 ):
     """
     Implements the fused computation for a Mixture of Experts (MOE) using
@@ -867,7 +871,7 @@ def _fused_moe_persistent_silu_kernel(
     # -----------------------------------------------------------
     # Simply compute how many iterations each persistent block needs to do
     start_pid = tl.program_id(axis=0)
-    NUM_XCDS: tl.constexpr = 8
+
     # Load tile-invariant runtime constant
     num_tokens_post_padded = tl.load(num_tokens_post_padded_ptr)
 
@@ -988,7 +992,7 @@ def _fused_moe_persistent_silu_kernel(
         silu_acc, mul_acc = (
             accumulator.to(tl.float32).reshape(BLOCK_SIZE_M, BLOCK_SIZE_HALF, 2).split()
         )
-        silu_acc = silu_acc / (1.0 + tl.exp2(-(silu_acc * 1.44269504089)))
+        silu_acc = _silu_exp2(silu_acc)
         accumulator = (silu_acc * mul_acc).to(compute_type)
 
         # -----------------------------------------------------------
@@ -1026,13 +1030,19 @@ def fused_moe_silu(
     """
     #TODO: Add doc
     """
+    _LOGGER.info(
+        f"FUSED_MOE_SILU:  A={tuple(A.shape)}  B={tuple(B.shape)}  C={tuple(C.shape)} "
+        + f"topk_weights={tuple(topk_weights.shape)} sorted_token_ids={tuple(sorted_token_ids.shape)} "
+        + f"expert_ids={tuple(expert_ids.shape)} num_tokens_post_padded={tuple(num_tokens_post_padded.shape)} "
+        + f"top_k={top_k} "
+    )
     assert topk_weights.stride(1) == 1
     assert sorted_token_ids.stride(0) == 1
 
     if use_fp8_w8a8:
         assert B_scale is not None
         if block_shape is None:
-            output = torch.zeros(A.shape, device=A.device, dtype=torch.float8_e4m3fnuz)
+            output = torch.zeros(A.shape, device=A.device, dtype=B.dtype)
             A_scale = torch.zeros(1, device=A.device, dtype=torch.float32)
             A, A_scale = _MOE_A_QUANT_FUNC(output, A, A_scale)
         else:
@@ -1111,6 +1121,7 @@ def fused_moe_silu(
                 has_zp=B_zp is not None,
                 use_int4_w4a16=use_int4_w4a16,
                 use_int8_w8a16=use_int8_w8a16,
+                NUM_XCDS=get_num_xcds(),
                 **config,
             )
         else:
@@ -1153,6 +1164,7 @@ def fused_moe_silu(
                 has_zp=B_zp is not None,
                 use_int4_w4a16=use_int4_w4a16,
                 use_int8_w8a16=use_int8_w8a16,
+                NUM_XCDS=get_num_xcds(),
                 **config,
             )
 
@@ -1201,6 +1213,7 @@ def fused_moe_silu(
                 compute_type=compute_type,
                 use_fp8_w8a8=use_fp8_w8a8,
                 use_int8_w8a16=use_int8_w8a16,
+                NUM_XCDS=get_num_xcds(),
                 **config,
             )
         else:
@@ -1241,5 +1254,6 @@ def fused_moe_silu(
                 compute_type=compute_type,
                 use_fp8_w8a8=use_fp8_w8a8,
                 use_int8_w8a16=use_int8_w8a16,
+                NUM_XCDS=get_num_xcds(),
                 **config,
             )
